unit UCustomIni;

interface
uses Classes, Inifiles, ActnList, SySutils, StdCtrls, UGenDeclarations;

type
  TCustomIni = class                              // Holds ini file configuration and
  private                                         // manages the MRU list
    FMaxMRUfiles: integer;
    Fini: TIniFile;                               // the ini file object
    FProfileList: TStringList;                    // the MRU list
    FDefaultExt: string;                          // default extention for config files
    FProfileControl: TComboBox;                   // the control to displaying the MRU list
    FUsesIniDefaults: boolean;                    // use HTML attributes stored in the ini file
    FIniDefsCheckBox: TCheckBox;                  // control for use ini defaults
    FSettingsAction: TSettingsAction;             // Action to enable/disable depending on who
    function GetIniFile: string;                  // did the update (user input --> object needs update)
    function GetCurrProfile: string;
    procedure SetUsesIniDEfaults(flag: boolean);
  public
    constructor create(const FileName: string; Action: TSettingsAction); overload;
    // assign screen controls to be updated
    procedure assignControls(ProfileControl: TComboBox; IniDefsCheck: TcheckBox);

    procedure ReadIniValues;                      // get current profile and MRU from ini
    procedure SaveIniValues;                      // save MRU to ini
    procedure AddMRU(const ProfileName: string);  // add a new file to the MRU list
    procedure clearMRU;                           // empty the MRU list
    destructor free;
  published
    property ini: TIniFile read Fini write Fini;
    property FileName: string read GetIniFile;
    property DefaultExt: string read FDefaultExt write FDefaultExt;
    property CurrProfileName: string read GetCurrProfile write AddMRU;
    property UsesIniDefaults: boolean read FUsesIniDefaults write SetUsesIniDefaults;
  end;
  {---------------------------------------}

implementation

(******************************************************************************************************)

constructor TCustomIni.create(const FileName: string; Action: TSettingsAction);
begin
  FIniDefsCheckBox := nil;                        // clear out the controls fields for checking
  FMaxMRUFiles := MaxMRU;                         // with assigned() to work
  Fini := TiniFile.Create(FileName);              // create the ini file object
  FProfilelist := TstringList.Create;             // create the profile list
  FProfilecontrol := nil;                         // this will be created elsewhere,
  FDefaultExt := ConfigExt;                       // we just need to assign it later
  if assigned(Action) then
    FSettingsAction := Action
  else
    FSettingsAction := nil
end;
(******************************************************************************************************)

destructor TCustomIni.free;
begin
  if assigned(Fini) then                          // free the ini object
    Fini.free;
  if assigned(FProfilelist) then                  // FprofileControl is freed bu the main form
    FProfilelist.free;
end;
(******************************************************************************************************)

procedure TCustomIni.assignControls(ProfileControl: TComboBox; IniDefsCheck: TcheckBox);
begin
  if assigned(IniDefsCheck) then
    FIniDefsCheckBox := IniDefsCheck;
  if assigned(ProfileControl) then
    FProfileControl := ProfileControl;
end;
(******************************************************************************************************)

procedure TCustomIni.ClearMRU;                    // clear the most recently used profile list
begin
  if Assigned(FProfileList) then
    begin
      FProfileList.clear;                         // clear the list
      if assigned(Fini) then
        Fini.EraseSection('Profiles');            // update the ini file
      if Assigned(FProfileControl) then
        fProfilecontrol.Items.Clear;              // update the screen control
      SaveIniValues;
      ReadIniValues;
    end;
end;
(******************************************************************************************************)

procedure TCustomIni.SetUsesIniDefaults(flag: boolean); // setter for FUsesIniDEfaults
begin                                             // also updates the screen control if assinged
  FUsesIniDefaults := flag;
  if assigned(FIniDefsCheckBox) then
    FIniDefsCheckBox.checked := flag;
end;
(******************************************************************************************************)

function TCustomIni.GetCurrProfile: string;       // return current profile
begin
  if FProfileList.Count > 0 then
    result := FprofileList.Strings[0]
  else
    result := extractFilePath(ParamSTR(0)) + 'Default' + FDefaultExt;
end;
(******************************************************************************************************)

function TCustomIni.GetIniFile: string;           // returns the ini file name
begin
  if assigned(Fini) then
    Result := Fini.FileName
  else
    Result := '';
end;
(******************************************************************************************************)

procedure TCustomIni.AddMRU(const ProfileName: string);
var oldIndex: integer;                            // ad a loaded profile to the MRU list
begin
  oldIndex := Fprofilelist.IndexOf(Profilename);
  if OldIndex <> -1 then                          // no duplicates in MRU
    FProfileList.Delete(OldIndex);
  FProfileList.insert(0, ProfileName);            // add files LIFO style to the MRU
  if assigned(FProfileControl) then
    try                                           // the MRU list can't be sorted so
      FProfileControl.Items.BeginUpdate;          // can't use TstringList.duplicates
      FProfileControl.Items.Assign(FProfileList); // update the screen control
    finally
      FProfileControl.Items.EndUpdate;
      FProfileControl.ItemIndex := 0;             // select the first entry (the current one0
      if Assigned(FSettingsAction) then
        FSettingsAction.SettingsChanged := false; // indicate that no changes have been made so far
    end;
end;
(******************************************************************************************************)

procedure TCustomIni.ReadIniValues;
var index: integer;                               // read profile information from ini file
  TempStr: string;
begin                                             // read profile information from ini file
  if Fini.SectionExists('Profiles') then
    for index := 0 to FMaxMRUFiles - 1 do         // read the MRU into the internal list
      begin
        TempStr := Fini.ReadString('Profiles', 'Profile #' + inttostr(index), '');
        if length(tempstr) > 0 then
          if FProfileList.IndexOf(tempstr) = -1 then
            FProfileList.Add(Tempstr);            // only add a profile once
      end;
  AddMRU(CurrProfileName);                        // if ini file is blank add the default profile
  // read flag indicationg where to get defaults for new files from
  UsesInidefaults := Fini.ReadBool('Defaults storage', 'Use ini defaults', false);
  if FusesIniDefaults then                        // if defaults are loaded from the ini file then
    begin                                         // read them
      DefBackground := Fini.ReadInteger('Defaults', 'HTML Background', DefBackGroundConst);
      DefText := Fini.ReadInteger('Defaults', 'HTML Text Color', DefTextConst);
      DefLink := Fini.ReadInteger('Defaults', 'HTML Link Color', DefLinkConst);
      DefVisitedLink := Fini.ReadInteger('Defaults', 'Visited Link color', DefVisitedLinkConst);
      ConfigExt := Fini.ReadString('Defaults', 'Default Profile Extention', ConfigExtConst);
    end;
  if Assigned(FSettingsAction) then
    FSettingsAction.SettingsChanged := false;     // indicate that no changes have been made so far
end;
(*************************************************************************************************)

procedure TCustomIni.SaveIniValues;
var index: integer;                               // save profile information to ini file
begin                                             // profilelist will only be saved in the ini file
  index := 0;
  while (index < FMaxMRUFiles - 1) and (index < FProfileList.Count) do // save profile list
    begin
      Fini.WriteString('Profiles', 'Profile #' + IntToStr(index), FProfileList.strings[index]);
      inc(index);
    end;                                          // save defaults and defaults location flag
  Fini.WriteBool('Defaults storage', 'Use ini defaults', FUsesIniDefaults);
  Fini.WriteInteger('Defaults', 'HTML Background', DefBackGround);
  Fini.WriteInteger('Defaults', 'HTML Text Color', DefText);
  Fini.WriteInteger('Defaults', 'HTML Link Color', DefLink);
  Fini.WriteInteger('Defaults', 'Visited Link color', DefVisitedLink);
  Fini.WriteString('Defaults', 'Default Profile Extention', ConfigExt);
end;
(*************************************************************************************************)

end.

