unit UHighLighter;
interface

uses Classes, StdCtrls, SysUtils, Windows, Forms, comctrls, ActnList, toolEdit,
  UAppOptions, UHTMLOptions, UIOOptions, UGenDeclarations;

type

  {---------------------------------------}
  THighlightProfile = class             // general class holding all configurable options
  private
    FPageOptions: THTMLOptions;         // HTML options
    FAppOptions: TAppOptions;           // application options
    FIOOptions: TIOOptions;             // Input/Output options
    FNameList: TNameHighlight;          // list of names to highlight
    FSettingsAction: TSettingsAction;
    FNameListBox: TListBox;
    FNameEdit: TEdit;
    FNameImgEdit: TFileNameEdit;
    FNameCol: Tlabel;
    FFrontMemo,
      FEndMemo: TMemo;
    FDeleteAction,
      FUpdateAction: TAction;
    procedure ReadNamesFromStream(InputBuffer: TMemoryStream);
    procedure WriteNamesToStream(OutputBuffer: TMemoryStream);
  public
    constructor create(action: TsettingsAction); overload;
    // update the Highlight names page only
    procedure SynchNamesPage;           // update UI controls in the names page
    procedure NamesListChange;          // the names list has changed
    procedure AddName(const Pname, Pimage, PcustomFrontHTML, PCustomEndHTML: string;
      Pcolor: THTMlcolor);
    procedure DeleteName(index: integer);
    procedure UpdateName(const Pname, Pimage, PcustomFrontHTML, PCustomEndHTML: string;
      Pcolor: THTMlcolor; index: integer);
    procedure assignControls(NameListBox: TListBox;
      SrcFilesList: TListView;
      NameEdit,
      TitleEdit,
      AuthEdit: TEdit;
      BackImageEdit,
      NameImgEdit: TFileNameEdit;
      NameCol,
      ShowBackCol,
      ShowTextCol,
      ShowLinkcol,
      ShowVisLinkcol: Tlabel;
      ActRemoveFromList,
      ActConvert,
      DeleteAction,
      UpdateAction: TAction;
      inputDirEdit,
      DestDirEdit,
      DEstImgEdit: TDirectoryEdit;
      RelImgCheck,
      BrokenLinesCheck,
      CaseSensCheck,
      PromptOnOverwCheck,
      SkipExistingCheck: TCheckBox;
      frontPageHTMLMemo,
      EndPageHTMlMemo,
      FrontMemo,
      EndMemo: TMemo);
    function LoadFromFile(const filename: string): boolean;
    function SaveToFile(const filename: string): boolean;
    destructor free;
  published
    property PageOptions: THTMLOptions read FPageOptions;
    property AppOptions: TAppOptions read FAppOptions;
    property NameList: TNameHighlight read FNameList;
    property IOOptions: TIOOptions read FIOOptions;
  end;


  (****************************************************************************************************)
implementation
(******************************************************************************************************)

constructor THighlightProfile.create(Action: TSettingsAction);
begin                                   // create member classes
  FPageOptions := THTMLOptions.Create(Action); // for HTML options
  FAppOptions := TappOptions.Create(Action); // for application options
  FNameList := TStringList.Create;      // for the names to be highlighted
  FIOOptions := TIOOptions.Create(Action); // for i/O options
  if Assigned(Action) then
    FSettingsAction := Action
  else
    FSettingsAction := nil;
  FNameListBox := nil;
  FNameEdit := nil;
  FNameImgEdit := nil;
  FNameCol := nil;
  FFrontMemo := nil;
  FEndMemo := nil;
  FDeleteAction := nil;
  FUpdateAction := nil;
end;
(******************************************************************************************************)

destructor THighlightProfile.free;
var
  i: integer;
begin
  if assigned(FNameList) then
    begin
      if FnameList.count > 0 then
        for i := 0 to FNameList.Count - 1 do // free attribute objects first
          if assigned(FNameList.objects[i]) then
            THighlightInfo(FNameList.objects[i]).free;
      NameList.free;                    // the clear the name list
    end;
  if assigned(FPageOptions) then
    FPageOptions.free;
  if assigned(FAppOptions) then
    FAppOptions.free;
  if assigned(FIOOPtions) then
    FIOOptions.free;
end;
(******************************************************************************************************)

procedure THighlightProfile.assignControls(
  NameListBox: TListBox;
  SrcFilesList: TListView;
  NameEdit, TitleEdit, AuthEdit: TEdit;
  BackImageEdit, NameImgEdit: TFileNameEdit;
  NameCol, ShowBackCol, ShowTextCol, ShowLinkcol, ShowVisLinkcol: Tlabel;
  ActRemoveFromList, ActConvert, DeleteAction, UpdateAction: TAction;
  inputDirEdit, DestDirEdit, DEstImgEdit: TDirectoryEdit;
  RelImgCheck, BrokenLinesCheck, CaseSensCheck, PromptOnOverwCheck, SkipExistingCheck: TCheckBox;
  frontPageHTMLMemo, EndPageHTMlMemo, FrontMemo, EndMemo: TMemo);
begin
  FNameListBox := NameListBox;
  FNameEdit := NameEdit;
  FNameImgEdit := NameImgEdit;
  FNameCol := NameCol;
  FFrontMemo := FrontMemo;
  FEndMemo := EndMemo;
  FDeleteAction := DeleteAction;
  FUpdateAction := UpdateAction;
  FAppOptions.CaseCheckBox := CaseSensCheck;
  FIOOptions.assignControls(SrcFilesList, inputDirEdit, DestDirEdit,
    DestImgEdit, PromptOnOverwCheck, SkipExistingCheck, ActRemoveFromList, ActConvert);
  FPageOptions.assignControls(TitleEdit, AuthEdit, BackImageEdit, RelImgCheck,
    BrokenLinesCheck, frontPageHTMLMemo, EndPageHTMlMemo, ShowBackCol, ShowTextCol,
    ShowLinkcol, ShowVisLinkcol);
end;
(******************************************************************************************************)

procedure THighlightProfile.AddName(const Pname, Pimage,
  PcustomFrontHTML, PCustomEndHTML: string; Pcolor: THTMlcolor);
var
  HighLightInfo: THighLightInfo;
  currIndex: integer;
begin
  CurrIndex := FNameList.indexof(Pname); // check for existing names
  if CurrIndex <> -1 then               // if name exists then
    // update its attributes
    UpdateName(Pname, Pimage, PcustomFrontHTML, PCustomEndHTML, Pcolor, Currindex)
  else
    begin                               // if the name was not found
      HighlightInfo := THighlightInfo.create; // create a new highlight object
      HighlightInfo.HColor := Pcolor;   // with the properties passed as parameters
      HighlightInfo.HImage := PImage;
      HighlightInfo.HCustomFrontHTML := PCustomFrontHTML;
      HighlightInfo.HCustomEndHTML := PCustomEndHTML;
      FNameList.AddObject(Pname, HighlightInfo);
    end;
  SynchNamesPage;
  if assigned(FnameListBox) then
    FNameListBox.ItemIndex := FnameList.Count - 1;
  FSettingsAction.SettingsChanged := true;
end;
(******************************************************************************************************)

procedure THighlightProfile.UpdateName(const Pname, Pimage, PcustomFrontHTML,
  PCustomEndHTML: string; Pcolor: THTMLcolor; index: integer);
var
  HighLightInfo: THighLightInfo;
begin
  if Index < FNameList.count then       // proceed if name exists
    begin
      if not assigned(FNameList.Objects[Index]) then // name exists but has no
        begin                           // associated highlight info
          HighlightInfo := THighlightInfo.create; // create a new highlight object
          HighlightInfo.HColor := Pcolor; // with the properties passed as parameters
          HighlightInfo.HImage := PImage;
          HighlightInfo.HCustomFrontHTML := PCustomFrontHTML;
          HighlightInfo.HCustomendHTML := PCustomEndHTML;
          FNameList.Objects[Index] := HighlightInfo; // add the object to the name list
        end
      else
        with THighlightInfo(FNameList.Objects[Index]) do // just change the highlight options
          begin                         // to the ones passed as parameters
            HImage := PImage;
            HCustomFrontHTML := PCustomFrontHTMl;
            HCustomEndHTML := PCustomEndHTMl;
            HColor := PColor;
          end;
      FNameList.Strings[Index] := Pname;
    end;
  SynchNamesPage;
  FSettingsAction.SettingsChanged := true;
end;
(******************************************************************************************************)

procedure THighlightProfile.DeleteName(index: integer); // remove the specified
begin                                   // entry and associated object
  if FNameList.Strings[index] <> '' then // if it exists
    begin
      if assigned(FNameList.Objects[index]) then
        THighlightInfo(FNameList.Objects[index]).free;
      FNameList.Delete(index);
    end;
  SynchNamesPage;
  FSettingsAction.SettingsChanged := true;
end;
(******************************************************************************************************)

procedure THighlightProfile.ReadNamesFromStream(InputBuffer: TMemoryStream);
var                                     // streaming is done one string at a time
  ListCount: integer;                   // to keep object management simple
  StrLength: integer;                   // when loading
  index: integer;
  PColor: THTMlColor;
  PName: string;
  PImage: string;
  PCustomFrontHTML: string;
  PCustomEndHTML: string;
begin
  InputBuffer.Read(ListCount, 4);       // read total number of names
  if ListCount > 0 then
    for index := 0 to ListCount - 1 do  // read each name along with its
      begin                             // highlighting attributes
        InputBuffer.Read(PColor, sizeof(Pcolor));

        InputBuffer.Read(StrLength, sizeof(strLength));
        setlength(Pname, strlength);
        InputBuffer.Read(PName[1], StrLength);

        InputBuffer.Read(StrLength, sizeof(strLength));
        setlength(PImage, strlength);
        InputBuffer.Read(PImage[1], StrLength);

        InputBuffer.Read(StrLength, sizeof(strLength));
        setlength(PCustomFrontHTML, strlength);
        InputBuffer.Read(PCustomFrontHTML[1], StrLength);
        InputBuffer.Read(StrLength, sizeof(strLength));
        setlength(PCustomEndHTML, strlength);
        InputBuffer.Read(PCustomEndHTML[1], StrLength);
        // add the name to the highlight list
        AddName(Pname, PImage, PCustomFrontHTML, PCustomEndHTML, PColor);
      end;
end;
(******************************************************************************************************)

procedure THighlightProfile.WriteNamesToStream(OutputBuffer: TMemoryStream);
var
  index: integer;
  NameCount: integer;
  strlength: integer;
begin
  NameCount := FNameList.count;
  OutputBuffer.Write(NameCount, sizeof(nameCount)); // save total number of names
  if NameCount > 0 then                 // if any names in the list
    for index := 0 to NameCount - 1 do  // proceed to save each one
      with ThighlightInfo(FNameList.Objects[index]) do
        begin
          OutputBuffer.Write(Hcolor, sizeof(HColor));

          strlength := length(Namelist.strings[index]); // along with its size
          OutputBuffer.Write(strlength, sizeof(strLength));
          OutputBuffer.Write((Namelist.strings[index])[1], strlength);

          strlength := length(HImage);  // and associated attributes
          OutputBuffer.Write(strlength, sizeof(strLength));
          OutputBuffer.Write(HImage[1], strlength);

          strlength := length(HCustomFrontHTML);
          OutputBuffer.Write(strlength, sizeof(strLength));
          OutputBuffer.Write(HCustomFrontHTML[1], strlength);
          strlength := length(HCustomEndHTML);
          OutputBuffer.Write(strlength, sizeof(strLength));
          OutputBuffer.Write(HCustomEndHTML[1], strlength);
        end
end;
(******************************************************************************************************)

function THighlightprofile.LoadFromFile(const filename: string): boolean;
var
  ReadBuffer: TMemoryStream;            // reads config data from a file
  FSize: integer;
  newpos: integer;
begin
  Result := true;
  ReadBuffer := nil;
  try
    ReadBuffer := TMemoryStream.create; // create a bufer in memory for fast reading
    try
      ReadBuffer.LoadFromFile(filename); // read the entire file into memory
      if ReadBuffer.size = 0 then       // check for zero size errors
        Result := False
      else
        begin
          ReadBuffer.seek(0, SoFromBeginning); // get file length descriptor (first 4 bytes)
          ReadBuffer.Read(FSize, sizeof(Fsize));
          if FSize <> REadBuffer.Size then // check for file size errors
            raise ESizeException.Create('');
          ReadBuffer.Seek(sizeof(pointer) * 5, sofromBeginning);

          FPageOptions.LoadFromStream(ReadBuffer); // parse HTML options

          Readbuffer.Seek(sizeof(pointer), soFromBeginning);
          Readbuffer.Read(newpos, sizeof(newpos));
          Readbuffer.seek(newpos, soFromBeginning);
          FAppOptions.LoadFromStream(ReadBuffer); // parse application options

          Readbuffer.Seek(sizeof(pointer) * 2, soFromBeginning);
          Readbuffer.Read(newpos, sizeof(newpos));
          Readbuffer.seek(newpos, soFromBeginning);
          FIOOptions.LoadFromStream(ReadBuffer); // parse IO options

          Readbuffer.Seek(sizeof(pointer) * 3, soFromBeginning);
          Readbuffer.Read(newpos, sizeof(newpos));
          Readbuffer.seek(newpos, soFromBeginning);
          ReadNamesFromStream(ReadBuffer); // parse name list and associated attributes
        end;
    except
      application.messagebox('Try a different file name, or delete the current one to create a new'
        + #13 + 'one with the default configuration after restarting the application',
        'Error reading from the specified file', mb_iconstop);
      result := false;
    end;
  finally
    ReadBuffer.Free;                    // destroy the temp buffer
    SynchNamesPage;
    if assigned(FSettingsAction) then
      FSettingsAction.SettingsChanged := false;
  end;
end;
(******************************************************************************************************)

function THighlightprofile.SaveToFile(const filename: string): boolean;
var
  OutputBuffer: TMemoryStream;          // saves configuration to a file
  FSize: integer;
  Currpos: integer;
begin
  result := true;
  OutPutBuffer := nil;
  try
    OutputBuffer := TMemoryStream.Create; // create a temp memory buffer for fast writing
    try
      OutputBuffer.Seek(sizeof(pointer) * 5, soFromCurrent); // skip the length descriptor

      FPageOptions.SaveToStream(OutputBuffer); // save HTMl options
      Currpos := OutputBuffer.Size;

      Outputbuffer.Seek(sizeof(pointer), soFromBeginning); // write the section length
      OutPutBuffer.Write(CurrPos, sizeof(CurrPos)); // to the file "header"
      OutputBuffer.Seek(Currpos, soFromBeginning); // and go back where we left

      FAppOptions.SaveToStream(OutputBuffer); // save Application options
      Currpos := OutputBuffer.Size;
      Outputbuffer.Seek(sizeof(pointer) * 2, soFromBeginning); // write the section length
      OutPutBuffer.Write(CurrPos, sizeof(CurrPos)); // to the file "header"
      OutputBuffer.Seek(Currpos, soFromBeginning); // and go back where we left

      FIOOptions.SaveToStream(OutputBuffer); // save input/output options
      Currpos := OutputBuffer.Size;
      Outputbuffer.Seek(sizeof(pointer) * 3, soFromBeginning); // write the section length
      OutPutBuffer.Write(CurrPos, sizeof(CurrPos)); // to the file "header"
      OutputBuffer.Seek(Currpos, soFromBeginning); // and go back where we left

      WriteNamesToStream(OutputBuffer); // save the highlighted names and highlight attributes
      Currpos := OutputBuffer.Size;
      Outputbuffer.Seek(sizeof(pointer) * 4, soFromBeginning); // write the section length
      OutPutBuffer.Write(CurrPos, sizeof(CurrPos)); // to the file "header"
      OutputBuffer.Seek(Currpos, soFromBeginning); // and go back where we left

      FSize := OutputBuffer.Size;       // save the file length descriptor for error checking
      OutputBuffer.Seek(0, soFromBeginning);
      OutputBuffer.Write(FSize, sizeof(FSize));

      OutputBuffer.SaveToFile(filename); // dump the buffer to a disk file
    except
      application.messagebox('Delete the corrupted file and try a different configuration',
        'Error writing to the configuration file', mb_iconstop);
      result := false;
    end;
  finally
    OutputBuffer.Free;                  // destroy the temp buffer
    if assigned(FSettingsAction) then
      FSettingsAction.SettingsChanged := false;
  end;
end;
(******************************************************************************************************)

procedure THighlightprofile.NamesListChange; // the names list has changed
var
  selIndex: integer;
begin
  if Assigned(FNameListBox) then
    begin
      selIndex := self.FNameListBox.ItemIndex; // save current index
      if selindex <> -1 then            // synchronize the Ui controls with
        begin                           // the values held internally
          FNameEdit.text := FNameList.strings[SelIndex];
          if assigned(FNameList.objects[SelIndex]) then
            with ThighlightInfo(FNameList.Objects[SelIndex]) do
              begin
                FFrontMemo.text := HcustomFrontHTML;
                FEndMemo.text := HcustomEndHTML;
                FNameImgEdit.Text := HImage;
                FNameCol.color := HColor;
              end;
        end;
      FDeleteAction.Enabled := FNameListBox.ItemIndex <> -1; // enable/disable buttons based on
      FUpdateAction.Enabled := FNameListBox.ItemIndex <> -1; // selected items
    end;
end;
(**********************************************************************************************)

procedure THighlightprofile.SynchNamesPage; // update UI controls in the names page
var
  Oldindex: integer;                    // from memory objects
begin
  if assigned(FNameListBox) then
    begin
      OldIndex := FNameListBox.ItemIndex;
      try
        FNameListBox.Items.BeginUpdate; // disable screen updates for the list
        FNameListBox.Items.Clear;       // clear old values
        FNameListBox.Items := FNameList; // update the list
      finally
        FNameListBox.ItemIndex := OldIndex; // re-enable screen updates
        FNameListBox.Items.EndUpdate;
        NamesListChange;
      end;
      if oldindex = -1 then
        begin
          FNameEdit.Text := '';
          FFrontMemo.Clear;
          FEndMemo.Clear;
          FNameImgEdit.Text := '';
          FNameCol.Color := DEfText;
        end
      else
        NamesListChange;
    end;
end;
(******************************************************************************************************)
end.

