unit UIOOptions;

interface
uses UGenDeclarations, ActnList, stdctrls, Comctrls, Classes, toolEdit, Registry,
  Windows, Sysutils;

type
  {---------------------------------------}

  TIOOptions = class(TOptionClass)      // general options for read/write operations
  private
    FSettingsAction: TsettingsAction;
    FRegistryInputDir: string;          // quake 3 dir from registry
    FInputFiles: TStringList;           // list of files to be processed
    FDefaultInputDir,                   // default source file directory
      FHTMLDir: string;                 // default output directory
    FImagesDir: string;                 // default directory for images aperaing on HTML
    FOverwritePrompt,                   // "prompt on file found" flag
      FSkipExisting: boolean;           // "skip existing files" flag
    FControlsAssigned: boolean;
    // list of screen controls to be updated
    FActRemove,                         // RemoveFromList action (enable/disable)
      FActConvert: Taction;             // Convert action (enable/disable)
    FInputFilesList: TListView;         // listView for the input files list
    FinputDirEdit,                      // default source files directory edit
      FDestDirEdit,                     // destination directory edit
      FDEstImgEdit: TDirectoryEdit;     // images destination directory edit
    FPromptOnOverCheck,                 // prompt on overwrite checkbox
      FSkipExistingCheck: TcheckBox;    // skip existing files checkbox

    procedure setResult(ResIndex: integer; ResValue: TioStatus); // setter and
    function getResult(Resindex: integer): TioStatus; // getter for the results array
    procedure SetInDir(Value: string);
    procedure SetHTMLDir(Value: string);
    procedure SetImagesDir(Value: string);
    procedure SetOverwritePrompt(Value: boolean);
    procedure SetSkipExisting(Value: boolean);
  public
    constructor create(Action: TSettingsAction); overload;
    procedure UpdateFromUI;
    // assign the screen controls
    procedure assignControls(InputFilesList: TListView; inputDirEdit, DestDirEdit,
      DEstImgEdit: TDirectoryEdit; PromptOnOverCheck, SkipExistingCheck: TcheckBox;
      ActRemove, ActConvert: TAction);
    procedure SynchFilesList;           // synchronize screen controls with the files list
    procedure clearFilesStatus;         // clear IO operations results and update screen
    procedure addFiles(Strings: tStrings); // add new files to the input list
    procedure addfile(item: string);    // add a single file to the input list
    procedure deletefile(index: integer); // delete a single file from the input list

    function GetCurrentFile: string;    // return the current profile filename

    procedure LoadFromStream(ReadBuffer: TMemoryStream); override;
    procedure SaveToStream(OutputBuffer: TMemoryStream); override;
    destructor free; override;
    // conversion results array (IO operations results)
    property ConvResults[ResIndex: integer]: TioStatus read getResult write setResult;
  published
    property InputFiles: TStringList read FInputFiles;
    property DefaultInputDir: string read FDefaultInputdir write SetInDir;
    property HTMLDir: string read FHTMLDir write SetHTMLDir;
    property ImagesDir: string read FImagesDir write SetImagesDir;
    property OverwritePrompt: boolean read FOverwritePrompt write SetOverwritePrompt;
    property SkipExisting: boolean read FSkipExisting write SetSkipExisting;
  end;

  (**************************************************************************************************)
implementation
(****************************************************************************************************)

constructor TIOOptions.create(Action: tSettingsAction);
var
  REG: TRegistry;
begin
  FInputFiles := Tstringlist.create;    // list of source text files
  FInputfiles.Sorted := true;
  FInputFiles.Duplicates := dupIgnore;
  REG := TRegistry.create;
  REG.rootkey := HKEY_LOCAL_MACHINE;
  if REG.OpenKeyReadOnly('Software\Id\Quake III Arena\') then // look for the default Quake 3
    FRegistryInputDir := REG.ReadString('INSTALLPATH') // install path in the registry
  else
    FRegistryInputDir := ExtractFilePath(paramstr(0)); // if not found default to app. directory
  FDefaultInputDir := FRegistryInputDir;
  REG.CloseKey;
  REG.Free;
  FHTMLDir := ExtractFilePath(paramstr(0)); // default to app. dir. for html files
  FImagesDir := HTMLDir;                // same for images
  FOverwritePrompt := true;             // prompt for replacing existing files
  FSkipExisting := true;                // do not replace existing files by default
  FInputFilesList := nil;               // blank out screen controls initially
  FinputDirEdit := nil;
  FDestDirEdit := nil;
  FDEstImgEdit := nil;
  FPromptOnOverCheck := nil;
  FSkipExistingCheck := nil;
  if Assigned(Action) then
    FsettingsAction := Action
  else
    FsettingsAction := nil;
  FControlsAssigned := false;
end;
(****************************************************************************************************)

destructor TIOOptions.free;
var
  i: integer;
begin
  if assigned(inputfiles) then          // free the IO result objects that have been created
    if FinputFiles.Count > 0 then
      for i := 0 to Finputfiles.count - 1 do
        if assigned(FInputFiles.Objects[i]) then
          begin
            IOResClass(FInputFiles.Objects[i]).free;
            InputFiles.Objects[i] := nil
          end;
  FInputfiles.Free;
end;
(****************************************************************************************************)

procedure TIOOPtions.assignControls(InputFilesList: TListView; inputDirEdit, DestDirEdit,
  DEstImgEdit: TDirectoryEdit; PromptOnOverCheck, SkipExistingCheck: TcheckBox;
  Actremove, ActConvert: TAction);
begin
  try
    if Assigned(InputFilesList) then
      FInputFilesList := InputFilesList; // assign screen controls to be updated
    FinputDirEdit := inputDirEdit;
    FDestDirEdit := DestDirEdit;
    FDEstImgEdit := DEstImgEdit;
    FPromptOnOverCheck := PromptOnOverCheck;
    FSkipExistingCheck := SkipExistingCheck;
    FActRemove := ActRemove;
    FActConvert := ActConvert;
    FControlsAssigned := true;
  except
    FControlsAssigned := false;
  end;
end;
(****************************************************************************************************)

procedure TIOOptions.UpdateFromUI;
begin
  if FcontrolsAssigned then
    begin
      FDefaultInputDir := FInputDiredit.Text;
      FHTMLDir := FDestDirEdit.Text;
      FImagesDir := FDestImgEdit.Text;
      FOverwritePrompt := FPromptOnOverCheck.Checked;
      FSkipExisting := FSkipExistingCheck.Checked;
    end;
end;
(****************************************************************************************************)

procedure TIOOptions.addfile(item: string);
begin
  FInputFiles.add(item);
  SynchFilesList;                       // update screen controls
  FSettingsAction.SettingsChanged := true; // changes have been made
end;
(****************************************************************************************************)

procedure TIOOptions.deletefile(index: integer);
begin
  if assigned(FinputFiles.objects[Index]) then
    begin
      IoResCLass(FinputFiles.objects[Index]).free;
      FinputFiles.objects[Index] := nil;
    end;                                // do not update screen controls
  FInputFiles.Delete(index);            // (must be done manually to avoid flicker)
  FSettingsAction.SettingsChanged := true; // changes have been made
end;
(****************************************************************************************************)

procedure TIOOPtions.addFiles(strings: Tstrings);
begin
  FInputFiles.AddStrings(strings);
  if assigned(FInputFilesList) then
    begin
      SynchFilesList;
      FSettingsAction.SettingsChanged := true; // indicate that changes have been made
    end;
end;
(****************************************************************************************************)

procedure TIOOptions.clearFilesStatus;
var
  loopIndex: integer;
  LoopMax: integer;
begin
  if FControlsAssigned then
    if FinputFilesList.Items.Count > 0 then // then update the Input files Display control
      try                               // with the default result
        LoopMax := FinputfilesList.items.count;
        FinputFilesList.items.BeginUpdate;
        for LoopIndex := 0 to LoopMax - 1 do // set the oprations results  to "Ready" (default)
          if FinputFilesList.Items.Item[LoopIndex].subItems.Count = 0 then
            FinputFilesList.Items.Item[LoopIndex].subItems.add(DefOpStatus)
          else
            FinputFilesList.Items.Item[LoopIndex].subItems[0] := DefOpStatus;
      finally
        FinputFilesList.items.Endupdate;
      end;
end;
(****************************************************************************************************)

function TIOOptions.GetCurrentFile: string; // return current profile filename
begin
  result := '';
  if assigned(FinputFiles) then
    if Finputfiles.Count > 0 then
      result := Finputfiles.Strings[0];
end;
(****************************************************************************************************)

procedure TIOOptions.SynchFilesList;    // update UI controls in the input files page
var
  index: integer;                       // from memory objects
  newItem: TListItem;
begin
  if FControlsAssigned then
    with FinputFilesList do             // update the file list first
      begin
        try
          Items.BeginUpdate;            // lock changes
          items.Clear;                  // clear and updtae
          for index := 0 to FInputFiles.Count - 1 do
            begin
              newitem := Items.Add;
              newitem.Caption := FInputFiles.Strings[index];
            end;
        finally
          Items.EndUpdate;              // unlock changes
        end;
        clearFilesStatus;
        if assigned(FactRemove) then    // enabled/disable actions based on selected items
          FactRemove.enabled := (items.count > 0) and (SelCount > 0);
        if assigned(FActConvert) then
          FActConvert.enabled := Items.Count > 0;
      end;
end;
(****************************************************************************************************)

procedure TIOOPtions.SetInDir(Value: string); // setter for default input files directory
begin
  FDefaultInputDir := value;
  if assigned(FinputDirEdit) then
    FinputDirEdit.text := value;
end;
(****************************************************************************************************)

procedure TIOOPtions.SetHTMLDir(Value: string); // setter for output files directory
begin
  FHTMLDir := value;
  if assigned(FDestDirEdit) then
    FDestDirEdit.text := value;
end;
(****************************************************************************************************)

procedure TIOOPtions.SetImagesDir(Value: string); // setter for images directory
begin
  FImagesDir := value;
  if assigned(FDestImgEdit) then
    FDestImgEdit.text := value;
end;
(****************************************************************************************************)

procedure TIOOPtions.SetOverwritePrompt(Value: boolean); // setter for "prompt for overwrite" flag
begin
  FOverwritePrompt := value;
  if assigned(FPromptOnOverCheck) then
    FPromptOnOverCheck.checked := value;
end;
(****************************************************************************************************)

procedure TIOOPtions.SetSkipExisting(Value: boolean); // setter for "skip existing files" flag
begin
  FSkipExisting := value;
  if assigned(FSkipExistingCheck) then
    begin
      FSkipExistingCheck.checked := value;
      if assigned(FpromptOnOverCheck) then
        FPromptOnOverCheck.Enabled := not value;
    end;
end;
(****************************************************************************************************)

procedure TIOOptions.LoadFromStream(ReadBuffer: TMemoryStream);
var
  strlength: integer;
  FileCount: integer;
  TempString: string;
begin
  ReadBuFfer.Read(FOverWritePrompt, sizeof(FOverWritePrompt)); // read the boolean variables first
  ReadBuFfer.Read(FSkipExisting, sizeof(FSkipExisting));

  SetOverwritePrompt(FoverWritePrompt); // update screen controls
  SetSkipExisting(FSkipExisting);

  ReadBuffer.Read(FileCount, sizeof(fileCount)); // if any files then read the name (and file path)
  if FileCount > 0 then                 // of each one
    begin
      ReadBuffer.Read(strlength, sizeof(strlength));
      setlength(TempString, strlength); // read the length of the string first
      ReadBuffer.Read(TempString[1], strlength); // set the new string length before reading it
      FInputFiles.Text := TempString;
    end;

  SynchFilesList;                       // update screen controls

  ReadBuffer.Read(strlength, sizeof(strlength));
  setlength(FDefaultInputDir, strlength); // read the Initial directory string
  ReadBuffer.Read(FDefaultInputDir[1], strlength);

  SetInDir(FDefaultInputdir);           // update screen controls

  ReadBuffer.Read(strlength, sizeof(strlength));
  setlength(FHTMLDir, strlength);       // read the output (HTML) directory
  ReadBuffer.Read(FHTMLDir[1], strlength);

  SetHTMLDir(FHTMLDir);                 // update screen controls

  ReadBuffer.Read(strlength, sizeof(strlength));
  setlength(FImagesDir, strlength);     // read the destination directory for HTML images
  ReadBuffer.Read(FImagesDir[1], strlength);

  SetImagesDir(FImagesDir);
end;
(****************************************************************************************************)

procedure TIOOptions.SaveToStream(OutputBuffer: TMemoryStream);
var
  strlength: integer;
  FileCount: integer;
begin
  OutputBuFfer.Write(FOverWritePrompt, sizeof(FOverWritePrompt)); // save the boolean variables first
  OutputBuFfer.Write(FSkipExisting, sizeof(FSkipExisting));

  FileCount := FInputFiles.Count;       // save the number of files selected for processing
  OutputBuffer.Write(FileCount, sizeof(FileCount)); // if any files then save the name (and file path)
  if FileCount > 0 then                 // of each one
    begin
      strlength := length(FInputfiles.Text); // store the length of the string first
      OutputBuffer.Write(strlength, sizeof(strlength));
      Outputbuffer.write(Finputfiles.Text[1], strlength);
    end;

  strlength := length(FDefaultInputDir); // store the Initial directory string
  Outputbuffer.Write(strlength, sizeof(strlength));
  OutputBuffer.Write(FDefaultInputDir[1], strlength);

  strlength := length(FHTMLDir);        // store the output (HTML) directory
  Outputbuffer.Write(strlength, sizeof(strlength));
  OutputBuffer.Write(FHTMLDir[1], strlength);

  strlength := length(FImagesDir);      // store the destination directory for HTML images
  Outputbuffer.Write(strlength, sizeof(strlength));
  OutputBuffer.Write(FImagesDir[1], strlength);
end;
(****************************************************************************************************)

procedure TIOOptions.setResult(ResIndex: Integer; ResValue: TioStatus);
var
  newStatus: IOResClass;                // setter for an IO operation result
begin
  if resIndex < FInputFiles.Count then
    begin
      if assigned(FInputFiles.Objects[resIndex]) then
        newstatus := IOResclass(FInputFiles.Objects[resIndex])
      else
        begin
          newStatus := IOresClass.create;
          FInputFiles.Objects[resIndex] := newstatus;
        end;
      newstatus.status := ResValue;
      if assigned(FInputFilesList) then
        begin                           // update screen if possible
          FInputFilesList.Items.Item[resIndex].SubItems[0] := newstatus.StrResult;
        end;
    end;
end;
(***************************************************************************************************)

function TIOOptions.getResult(ResIndex: Integer): TioStatus; // return an io operation result
begin
  result := ioNone;
  if resIndex < FInputFiles.Count then
    if assigned(FInputFiles.Objects[resIndex]) then
      result := IOResClass(FInputFiles.Objects[resIndex]).status;
end;
(***************************************************************************************************)
end.

